***********************************************************************
*    Calculates # of points, maximum, minimum, mean, standard deviation,
*        and standard error of a numeric database field based on
*	 optional condition on one pass of the data.
*
*    Preconditions:  An array[6] must be passed to receive calculated values.
*
*    Released into the public domain.
*
*    Author - Walter M. Sydoriak,  5-14-89
***********************************************************************


* remove comments between TOP and BOTTOM below to run demo

***********************************  TOP  *****************************
*DECLARE stats[6]
*SET DECIMALS TO 10
*SET FIXED ON
*USE stdtest
*GO TOP
*? 'Data:'
*? 'Trees | Location'
*FOR I = 1 TO RECCOUNT()
*	? STR(trees,5) + ' | ' + location
*	SKIP
*NEXT i
*?
*success = std('trees',stats,'location = "VALLEY"')
*? 'Descriptives based on location = "VALLEY":'
*? stats[1]
*?? '  # of Points'
*? stats[2]
*?? '  Maximum'
*? stats[3]
*?? '  Minimum'
*? stats[4]
*?? '  Mean'
*? stats[5]
*?? '  Standard Deviation'
*? stats[6]
*?? '  Standard Error'
*?
*? 'Successful: ' + IF(success,'Yes', 'No')
*********************************  BOTTOM  ****************************



*!*********************************************************************
*!
*!       Function: STD()
*!
*!*********************************************************************
FUNCTION std

* para stdfield, stdarr, condition
*	- stdfield is name of field to get stats
*	- stdarr is array[6] for calculated values,
*	     stdarr[1] is # of points
*	     stdarr[2] is maximum
*	     stdarr[3] is minimum
*	     stdarr[4] is mean
*	     stdarr[5] is standard deviation
*	     stdarr[6] is standard error of the mean
*	- condition string (use '.t.' for no condition)
*
* Calls to: Stdmaxmin() contained in this listing
*
* Notes:
*
*	standard deviation formula is based on observations - 1
*
*	standard error formula is:
*	     standard deviation / square root(observations)
*
*	if field passed is not numeric or array not defined
*	     returns .f.
*
*	if observations = 0
*	     stdarr[1] = 0
*	     stdarr[2] = 0
*	     stdarr[3] = 0
*	     stdarr[4] = 0
*	     stdarr[5] = -1
*	     stdarr[6] = -1
*	     returns .f.
*
*	if observations = 1
*	     stdarr[1] = 1
*	     stdarr[2] = stdfield value
*	     stdarr[3] = stdfield value
*	     stdarr[4] = stdfield value
*	     stdarr[5] = -1
*	     stdarr[6] = -1
*	     returns .t.


PARA stdfield, stdarr, condition
PRIVATE mrec, sumsq, sums, obs, ret_val, tmp, first_time

* is field numeric and array defined
IF TYPE("&stdfield") # "N" .OR. TYPE("stdarr") # "A"
	RETURN .F.
ENDIF

* record to return to after calculations
m->mrec = RECNO()
GO TOP
stdarr[2] = 0
m->first_time = .T.

SUM &stdfield ** 2, &stdfield, 1, stdmaxmin() TO m->sumsq, m->sums, m->obs, m->tmp FOR &condition

* return to original record
GO m->mrec

* return value assumed ok
m->ret_val = .T.

IF m->obs = 0
	Afill(stdarr,0)
	stdarr[5] = -1
	stdarr[6] = -1
	m->ret_val = .F.
ELSEIF m->obs = 1
	stdarr[1] = 1
	stdarr[4] = m->sums / m->obs
	stdarr[5] = -1
	stdarr[6] = -1
	m->ret_val = .F.
ELSE
	stdarr[1] = m->obs
	stdarr[4] = m->sums / m->obs
	stdarr[5] = SQRT(((m->obs * m->sumsq) - (m->sums ** 2)) / (m->obs * (m->obs - 1)))
	stdarr[6] = stdarr[5] / SQRT(obs)
	m->ret_val = .T.
ENDIF
RETURN m->ret_val


FUNCTION stdmaxmin
stdarr[2] = MAX(&stdfield, stdarr[2])
IF m->first_time
	stdarr[3] = &stdfield
	m->first_time = .F.
ELSE
	stdarr[3] = MIN(&stdfield, stdarr[3])
ENDIF
RETURN 0
